#version 430 core

#include "global.inc"

// ----------------------------------
// default shader for everything
// 
// Available defines:
// - DIFFUSE
// - NORMAL
// - TRANSPARENT | CUTOUT
// ----------------------------------

in VertexOutput {
	vec3 positionWorld;
	vec3 normal;

	#ifdef NORMAL
		vec3 tangent;
		vec3 bitangent;
		mat3 tangentSpace;
	#endif

	vec2 uv0;

	#ifdef LIGHTMAP
		vec2 uv1;
	#endif

	#ifdef SHADOW
		vec4 lightSpacePos[NUM_SHADOWMAPS];
	#endif

} vertexOutput;


// ----------------------------------
// Uniforms
// ----------------------------------

uniform vec4 albedo;	// Main color. Alpha parameter controls transparency (if TRANSPARENT) or cutout level (if CUTOUT)
uniform float metallic;
uniform float roughness;

#ifdef DIFFUSE
	uniform sampler2D diffuseTex;
#endif

#ifdef NORMAL
	uniform sampler2D normalTex;
#endif


// ----------------------------------
// Local values
// ----------------------------------

out vec4 fragColor;

void main() 
{
	Surface surf;
	surf.albedo = albedo.rgb;
	surf.metallic = metallic;
	surf.roughness = roughness;
	surf.position = vertexOutput.positionWorld.xyz;
	surf.view = normalize(camera.position - surf.position);

	#ifdef SHADOW
		surf.lightSpacePos = vertexOutput.lightSpacePos;
	#endif

	#ifdef LIGHTMAP
		surf.lightMapUv = vertexOutput.uv1;
	#endif

	float alpha = 1.0;
	#ifdef TRANSPARENT
		alpha = albedo.a;
	#endif

	#ifdef DIFFUSE
		vec4 diffuse = texture(diffuseTex, vertexOutput.uv0);
		surf.albedo *= diffuse.rgb;

		#ifdef CUTOUT
			if (diffuse.a < albedo.a)
				discard;
		#endif

		#ifdef TRANSPARENT
			alpha *= diffuse.a;
		#endif
	#endif

	vec3 normal = normalize(vertexOutput.normal);
	#ifdef NORMAL
		if (normals){
			normal = texture(normalTex, vertexOutput.uv0).rgb;
			normal = normalize(normal * 2.0f - 1.0f);   
			normal = normalize(vertexOutput.tangentSpace * normal); 
		}
	#endif
	surf.normal = normal;	

	vec3 shaded = shade(surf);

	shaded *= brightness;

	fragColor = vec4(shaded, alpha);
	
	
	//fragColor = vec4(surf.albedo, alpha);



	//#ifdef FOG
	///float distance = length(posWorld - camera.position);
	// finalColor = calcFog(finalColor, distance);
	//#endif
}

