
// --------------------------------------------
// Global params
// --------------------------------------------

uniform float brightness;
uniform bool normals;



// --------------------------------------------
// Uniforms
// --------------------------------------------

uniform struct Frame
{
	float dt;
	float totalTime;
} frame;


uniform struct Camera
{
	float near;
	float far;
	vec3 position;
	mat4 eye;
	mat4 view;
	mat4 projection;
	mat4 viewProjection;
} camera;

uniform struct Viewport
{
	float x;
	float y;
	float width;
	float height;
} viewport;

// Per object matrices
uniform struct Object
{
	mat4 world;
	mat4 normal;
	mat4 worldView;
	mat4 worldViewProjection;
} object;

// set after depth prepass
uniform sampler2D depthTex;

// if a skybox is present
uniform samplerCube skybox;


// --------------------------------------------
// Utils
// --------------------------------------------

float restoreZ(float depth)
{
	// http://web.archive.org/web/20130416194336/http://olivers.posterous.com/linear-depth-in-glsl-for-real
	return 2.0f * camera.near * camera.far / (camera.far + camera.near - (2.0 * depth - 1.0) * (camera.far - camera.near));
}

/*
float mapRange(float qMin, float qMax, float pMin, float pMax, float value)
{
	return (pMax - pMin) * (value - qMin) / (qMax - qMin) + pMin;
}
*/

// map from u to x
float mapRange(float u1, float u2, float x1, float x2, float value)
{
	return value * ((x2 - x1) / (u2 - u1)) + (x1 * u2 - x2 * u1) / (u2 - u1);
}

vec2 mapVec2(vec2 u1, vec2 u2, vec2 x1, vec2 x2, vec2 value)
{
	return value * ((x2 - x1) / (u2 - u1)) + (x1 * u2 - x2 * u1) / (u2 - u1);
}



#include "lighting.inc"