#version 430 core

#include "global.inc"

in VertexOutput {
	vec3 positionWorld;
	vec3 normal;
	vec3 tangent;
	vec3 bitangent;
	mat3 tangentSpace;
	vec2 uv0;
	vec4 clip;
} vertexOutput;

uniform	sampler2D normalMap;

uniform vec3 color;
uniform float alpha;
uniform float fadeDistance;
uniform	float speed;
uniform	float tiling;

out vec4 fragColor;


void main() 
{
	vec3 posWorld = vertexOutput.positionWorld.xyz;
	vec3 normal = normalize(vertexOutput.normal);
	vec3 viewDir = normalize(camera.position - posWorld);
	
	/*
	vec2 uv0 = (vertexOutput.uv0 * tiling) + vec2(speed * frame.totalTime);
	normal = texture(normalMap, uv0).rgb;
	normal = normalize(normal * 2.0f - 1.0f);   
	normal = normalize(vertexOutput.tangentSpace * normal); 

	vec2 uv1 = (vec2(vertexOutput.uv0.y, - vertexOutput.uv0.x) * tiling) + vec2(speed * frame.totalTime);
	vec3 normal2 = texture(normalMap, uv1).rgb;
	normal2 = normalize(normal2 * 2.0f - 1.0f);   
	normal2 = normalize(vertexOutput.tangentSpace * normal2); 

	normal += normal2;
	normalize(normal);
	*/

	vec3 reflection = reflect(-viewDir, normal);
	vec3 skyboxRefl = texture(skybox, reflection).rgb;

	vec3 finalColor = mix(skyboxRefl, color, 0.3f);
	//finalColor *= calcLighting(posWorld, normal);
	
	float fragDepth = gl_FragCoord.z;
	float fragZ = restoreZ(fragDepth);
	
	vec2 screenUv = (vertexOutput.clip.xy / vertexOutput.clip.z + 1.0f) * 0.5f;
	float sceneDepth = texture(depthTex, screenUv).r;
	float sceneZ = restoreZ(sceneDepth);

	float distance = sceneZ - fragZ;
	float finalAlpha = mapRange(0, fadeDistance, 0, 1, distance);
	finalAlpha = clamp(finalAlpha, 0.0, 1.0) * alpha;

	// fade out over distance
	float start = 170.0;
	float range = 20.0;
	float camDistance = length(posWorld - camera.position);
	float amount = clamp((camDistance - start) / range, 0.0, 1.0);
	finalAlpha *= (1 - amount);

	fragColor = vec4(finalColor, finalAlpha);
}

