#version 430 core

#include "global.inc"

in VertexOutput 
{
	vec4 positionWorld;
	vec3 normal;
	vec2 uv;
} vertexOutput;

uniform struct Material
{
	vec3 color;
	float alpha;
	sampler2D diffuse;
	bool useDiffuse;
} material;

uniform sampler2D depthTex;


out vec4 fragColor;

float linearize_depth(float original_depth) {
    float near = 0.1;
    float far = 1000.0;
    return (2.0 * near) / (far + near - original_depth * (far - near));
}

void main() 
{
	vec3 posWorld = vertexOutput.positionWorld.xyz;
	vec3 normal = normalize(vertexOutput.normal);
	vec3 viewDir = normalize(camera.position - posWorld);

	//vec3 baseColor = vec3(0, 0, 0);	// ambient
	vec3 baseColor = material.color;

	#ifdef DIFFUSEMAP
	if (material.useDiffuse)
	{
		baseColor *= texture(material.diffuse, vertexOutput.uv).rgb;
	}
	#endif


	vec3 finalColor = vec3(0, 0, 0);
	
	for (int i = 0; i < NUM_LIGHTS; i++){

		Light light = lights[i];

		vec3 lightDir;
		if (light.type == LightTypeDirectional)
		{
			lightDir = -light.direction;
		} else 
		{
			lightDir = light.position - posWorld;
		}

		float distance = length(lightDir);
		lightDir = normalize(lightDir);

		float attenuation = light.intensity;
		if (light.type == LightTypePoint)
		{
			attenuation *= 1.0 / (light.constant + (light.linear * distance) +  (light.quadratic * pow(distance, 2)));
		}

		if (light.type == LightTypeSpot)
		{
			float cosInner = cos(light.innerAngle);
			float cosOuter = cos(light.outerAngle);
			attenuation *= clamp(smoothstep(cosOuter, cosInner, dot(-lightDir, normalize(light.direction))), 0.0f, 1.0f);
		}

		float NoL = max(dot(normal, lightDir), 0);
		vec3 R = normalize(reflect(-lightDir, normal));
		float RoV = max(dot(R, viewDir), 0);

		float tmpAlpha = 2.0f;
		finalColor += baseColor * (light.color * light.intensity) * (NoL + vec3(1, 1, 1) * pow(RoV, tmpAlpha)) * attenuation;
		//finalColor += baseColor * (light.color * NoL) * attenuation;
	}

	//finalColor *= brightness;
	fragColor = vec4(0.1, 0.1, 0.1, 1) + vec4(finalColor, 1);
}

