#version 430 core

#include "global.inc"

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;

#ifdef NORMAL
	layout(location = 2) in vec3 tangent;
	layout(location = 3) in vec3 bitangent;
#endif

layout(location = 4) in vec2 uv0;
layout(location = 5) in vec3 offset;

out VertexOutput 
{
	vec3 positionWorld;
	vec3 normal;

#ifdef NORMAL
	vec3 tangent;
	vec3 bitangent;
	mat3 tangentSpace;
#endif

	vec2 uv0;

#ifdef SHADOW
	vec4 lightSpacePos[NUM_SHADOWMAPS];
#endif

} vertexOutput;


void main() 
{
	vertexOutput.positionWorld = (object.world * vec4(position + offset, 1.0f)).xyz;
	vertexOutput.normal = (transpose(inverse(object.world)) * vec4(normal, 0.0f)).xyz;

#ifdef NORMAL
	vertexOutput.tangent = (object.world * vec4(tangent, 0.0f)).xyz;
	vertexOutput.bitangent = (object.world * vec4(bitangent, 0.0f)).xyz;
	vertexOutput.tangentSpace = mat3(vertexOutput.tangent, vertexOutput.bitangent, vertexOutput.normal);
#endif

	vertexOutput.uv0 = uv0;

#ifdef SHADOW
	for (int i = 0; i < NUM_SHADOWMAPS; i++)
	{
		vertexOutput.lightSpacePos[i] = lightViewProjection[i] * object.world * vec4(position, 1);
	}
#endif

	gl_Position = object.worldViewProjection * vec4(position + offset, 1);
}