#version 430 core

#include "global.inc"

in VertexOutput {
	vec3 positionWorld;
	vec3 normal;
	vec3 tangent;
	vec3 bitangent;
	mat3 tangentSpace;
	vec2 uv0;
	vec4 clip;

	#ifdef SHADOW
		vec4 lightSpacePos[NUM_SHADOWMAPS];
	#endif

} vertexOutput;

uniform	sampler2D normalMap;

uniform vec3 color;
uniform float alpha;
uniform float fadeDistance;
uniform	float speedU;
uniform	float speedV;
uniform	float tiling;
uniform	float normalStrength;
uniform float waterFogDistance;
uniform float waterFogRange;

out vec4 fragColor;


void main() 
{
	vec3 posWorld = vertexOutput.positionWorld.xyz;
	vec3 normal = normalize(vertexOutput.normal);
	vec3 viewDir = normalize(camera.position - posWorld);
	
	if (normals){
		vec2 uv0 = (vertexOutput.uv0 * tiling) + vec2(speedU, speedV) * frame.totalTime;
		vec3 normal1 = texture(normalMap, uv0).rgb;
		normal1 = normalize(normal1 * 2.0f - 1.0f);   
		normal1 = normalize(vertexOutput.tangentSpace * normal1); 

		vec2 uv1 = (vec2(vertexOutput.uv0.y, - vertexOutput.uv0.x) * tiling) + vec2(speedU, speedV) * frame.totalTime;
		vec3 normal2 = texture(normalMap, uv1).rgb;
		normal2 = normalize(normal2 * 2.0f - 1.0f);   
		normal2 = normalize(vertexOutput.tangentSpace * normal2); 

		normal = normal + (normal1 + normal2) * normalStrength;
		normalize(normal);
	}

	/*
	Surface surf;
	surf.albedo = color.rgb;
	surf.metallic = 1;
	surf.roughness = 0.7;
	surf.position = posWorld;
	surf.view = viewDir;
	surf.normal = normal;

	#ifdef SHADOW
		surf.lightSpacePos = vertexOutput.lightSpacePos;
	#endif
	vec3 shaded = shade(surf);
	*/
	
	vec3 reflection = reflect(-viewDir, normal);
	vec3 skyboxRefl = texture(skybox, reflection).rgb;

	vec3 finalColor = mix(skyboxRefl, color, 0.3f);
	//finalColor *= calcLighting(posWorld, normal);
	//finalColor *= 1.2;
	finalColor *= brightness;
	
	float fragDepth = gl_FragCoord.z;
	float fragZ = restoreZ(fragDepth);
	
	vec2 screenUv = (vertexOutput.clip.xy / vertexOutput.clip.z + 1.0f) * 0.5f;

	//vec2 screenUv = vertexOutput.clip.xy / vertexOutput.clip.z;
	//screenUv = mapVec2(vec2(-1, -1), vec2(1, 1), vec2(viewport.x, viewport.width), vec2(viewport.y, viewport.height), screenUv);

	float sceneDepth = texture(depthTex, screenUv).r;
	float sceneZ = restoreZ(sceneDepth);

	float distance = abs(sceneZ - fragZ);
	//float finalAlpha = mapRange(0, fadeDistance, 0, 1, distance);
	//finalAlpha = clamp(finalAlpha, 0, 1.0);
	//finalAlpha *= alpha;

	float finalAlpha = clamp(distance / fadeDistance, 0.0, 1.0) * alpha;
	//finalAlpha = clamp(alpha, 0, 1.0);

	// fade out over distance
	float start = waterFogDistance; //170.0;
	float range = waterFogRange;
	float camDistance = length(posWorld - camera.position);
	float amount = clamp((camDistance - start) / range, 0.0, 1.0);
	finalAlpha *= (1 - amount);
	

	fragColor = vec4(finalColor, finalAlpha);
	
}

