#version 430 core

#include "global.inc"
#define MAX_WAVES 7

layout(location = 0) in vec3 position;
// layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv0;

struct Wave {
float amplitude;
float frequency;
float phase;
vec3 dir;
};

uniform Wave waves[MAX_WAVES];

out VertexOutput {
	vec3 positionWorld;
	vec3 normal;
	vec3 tangent;
	vec3 bitangent;
	mat3 tangentSpace;
	vec2 uv0;
	vec4 clip;

	#ifdef SHADOW
		vec4 lightSpacePos[NUM_SHADOWMAPS];
	#endif

} vertexOutput;


void main() {


	vec3 newPosition = position;
	for (int i = 0 ; i < MAX_WAVES; i++){
	newPosition.y += waves[i].amplitude*sin(waves[i].frequency*(waves[i].dir.x*position.x+waves[i].dir.y*position.z)+frame.totalTime*waves[i].phase);
	}

	vec3 tangent = vec3(1,0,0);
	for (int i = 0; i < MAX_WAVES; i++){
		tangent.y += waves[i].amplitude*cos(waves[i].frequency*(waves[i].dir.x*position.x+waves[i].dir.y*position.z)+frame.totalTime*waves[i].phase)*
			waves[i].frequency*waves[i].dir.x;
	}
	tangent = normalize(tangent);
	vec3 bitangent = vec3(0,0,1);
	for (int i = 0; i < MAX_WAVES; i++){
		bitangent.y += waves[i].amplitude*cos(waves[i].frequency*(waves[i].dir.x*position.x+waves[i].dir.y*position.z)+frame.totalTime*waves[i].phase)*
			waves[i].frequency*waves[i].dir.y;
	}
	bitangent = normalize(bitangent);

	vec3 newNormal = normalize(-cross(tangent,bitangent));

	vertexOutput.positionWorld = (object.world * vec4(newPosition, 1.0f)).xyz;
	vertexOutput.normal = (transpose(inverse(object.world)) * vec4(newNormal, 0.0f)).xyz;
	vertexOutput.tangent = (object.world * vec4(tangent, 0.0f)).xyz;
	vertexOutput.bitangent = (object.world * vec4(bitangent, 0.0f)).xyz;
	vertexOutput.tangentSpace = mat3(vertexOutput.tangent, vertexOutput.bitangent, vertexOutput.normal);
	vertexOutput.uv0 = uv0;
	
	#ifdef SHADOW
		for (int i = 0; i < NUM_SHADOWMAPS; i++)
		{
			vertexOutput.lightSpacePos[i] = lightViewProjection[i] * object.world * vec4(position, 1);
		}
	#endif

	vec4 pos = object.worldViewProjection * vec4(newPosition, 1);
	//vertexOutput.screenUv = (pos.xy / pos.w) + 1.0f, pos.y / pos.w + 1.0f) * 0.5f;
	//vertexOutput.screenUv = (pos.xy / pos.w) * 0.5f + 0.5f;
	vertexOutput.clip = pos;

	gl_Position = object.worldViewProjection * vec4(newPosition, 1);
}